static const struct { char *name; TokenType value; } token_list[] = {
  { "+",        TOK_PLUS        },
  { "-",        TOK_MINUS       },
  { "*",        TOK_MULTIPLY    },
  { "/",        TOK_DIVIDE      },
  { "%",        TOK_MOD         },
  { "&",        TOK_BITWISE_AND },
  { "|",        TOK_BITWISE_OR  },
  { "^",        TOK_BITWISE_XOR },
  { "~",        TOK_BITWISE_NOT },

  { "if",       TOK_IF          },
  { "switch",   TOK_SWITCH      },
  { "default",  TOK_DEFAULT     },

  { "and",      TOK_AND         },
  { "or",       TOK_OR          },
  { "xor",      TOK_XOR         },
  { "not",      TOK_NOT         },

  { "union",     TOK_UNION     },
  { "intersect", TOK_INTERSECT },

  { "assign",   TOK_ASSIGN      },

  { "ends_block",                 TOK_ENDS_BLOCK                 },
  { "next_block_dynamic",         TOK_NEXT_BLOCK_DYNAMIC         },
  { "dont_postincdec_unexpanded", TOK_DONT_POSTINCDEC_UNEXPANDED },
  { "skip_two_operand_words",     TOK_SKIP_TWO_OPERAND_WORDS     },
  { "skip_four_operand_words",    TOK_SKIP_FOUR_OPERAND_WORDS    },

  { "skip_one_pointer",  TOK_SKIP_ONE_POINTER  },
  { "skip_two_pointers", TOK_SKIP_TWO_POINTERS },
  { "native_code",       TOK_NATIVE_CODE       },

  { "ccc",    TOK_CCC  },
  { "ccn",    TOK_CCN  },
  { "ccv",    TOK_CCV  },
  { "ccx",    TOK_CCX  },
  { "ccnz",   TOK_CCNZ },

  { ">>",       TOK_SHIFT_RIGHT },
  { "<<",       TOK_SHIFT_LEFT },

  { "=",        TOK_EQUAL            },
  { "<>",       TOK_NOT_EQUAL        },
  { ">",        TOK_GREATER_THAN     },
  { "<",        TOK_LESS_THAN        },
  { ">=",       TOK_GREATER_OR_EQUAL },
  { "<=",       TOK_LESS_OR_EQUAL    },

  { "true",     TOK_TRUE        },
  { "false",    TOK_FALSE       },

  { "numargs",  TOK_NUMARGS     },

  { "error",    TOK_ERROR },

  { "list",     TOK_EXPLICIT_LIST },

  { "nil",      TOK_NIL         },

  { "call",     TOK_FUNC_CALL },
  { "cast",     TOK_CAST      },
  { "code",     TOK_CODE      },

  { "deref",   TOK_DEREF },
  { "derefsb", TOK_DEREF },   /* Processed more fully in token.c */
  { "derefub", TOK_DEREF },
  { "derefsw", TOK_DEREF },
  { "derefuw", TOK_DEREF },
  { "derefsl", TOK_DEREF },
  { "dereful", TOK_DEREF },

  { "swapsb", TOK_SWAP },   /* Processed more fully in token.c */
  { "swapub", TOK_SWAP },
  { "swapsw", TOK_SWAP },
  { "swapuw", TOK_SWAP },
  { "swapsl", TOK_SWAP },
  { "swapul", TOK_SWAP },

  { "+tail",     TOK_TAIL      },
  { "define",    TOK_DEFINE    },
  { "defopcode", TOK_DEFOPCODE },
  { "include",   TOK_INCLUDE   },

  { "tmp.ub", TOK_TEMP_REGISTER },
  { "tmp.uw", TOK_TEMP_REGISTER },
  { "tmp.ul", TOK_TEMP_REGISTER },
  { "tmp.sb", TOK_TEMP_REGISTER },
  { "tmp.sw", TOK_TEMP_REGISTER },
  { "tmp.sl", TOK_TEMP_REGISTER },

  { "tmp2.ub", TOK_TEMP_REGISTER },
  { "tmp2.uw", TOK_TEMP_REGISTER },
  { "tmp2.ul", TOK_TEMP_REGISTER },
  { "tmp2.sb", TOK_TEMP_REGISTER },
  { "tmp2.sw", TOK_TEMP_REGISTER },
  { "tmp2.sl", TOK_TEMP_REGISTER },

  { "tmp3.ub", TOK_TEMP_REGISTER },
  { "tmp3.uw", TOK_TEMP_REGISTER },
  { "tmp3.ul", TOK_TEMP_REGISTER },
  { "tmp3.sb", TOK_TEMP_REGISTER },
  { "tmp3.sw", TOK_TEMP_REGISTER },
  { "tmp3.sl", TOK_TEMP_REGISTER },

  { "tmp4.ub", TOK_TEMP_REGISTER },
  { "tmp4.uw", TOK_TEMP_REGISTER },
  { "tmp4.ul", TOK_TEMP_REGISTER },
  { "tmp4.sb", TOK_TEMP_REGISTER },
  { "tmp4.sw", TOK_TEMP_REGISTER },
  { "tmp4.sl", TOK_TEMP_REGISTER },

  /* Dollar identifiers.  I put these here because these are somewhat annoying
   * to parse.
   */
  { "$1.dul", TOK_DOLLAR_DATA_REGISTER },
  { "$2.dul", TOK_DOLLAR_DATA_REGISTER },
  { "$3.dul", TOK_DOLLAR_DATA_REGISTER },
  { "$4.dul", TOK_DOLLAR_DATA_REGISTER },
  { "$5.dul", TOK_DOLLAR_DATA_REGISTER },
  { "$6.dul", TOK_DOLLAR_DATA_REGISTER },
  { "$7.dul", TOK_DOLLAR_DATA_REGISTER },
  { "$8.dul", TOK_DOLLAR_DATA_REGISTER },
  { "$9.dul", TOK_DOLLAR_DATA_REGISTER },

  { "$1.dsl", TOK_DOLLAR_DATA_REGISTER },
  { "$2.dsl", TOK_DOLLAR_DATA_REGISTER },
  { "$3.dsl", TOK_DOLLAR_DATA_REGISTER },
  { "$4.dsl", TOK_DOLLAR_DATA_REGISTER },
  { "$5.dsl", TOK_DOLLAR_DATA_REGISTER },
  { "$6.dsl", TOK_DOLLAR_DATA_REGISTER },
  { "$7.dsl", TOK_DOLLAR_DATA_REGISTER },
  { "$8.dsl", TOK_DOLLAR_DATA_REGISTER },
  { "$9.dsl", TOK_DOLLAR_DATA_REGISTER },

  { "$1.duw", TOK_DOLLAR_DATA_REGISTER },
  { "$2.duw", TOK_DOLLAR_DATA_REGISTER },
  { "$3.duw", TOK_DOLLAR_DATA_REGISTER },
  { "$4.duw", TOK_DOLLAR_DATA_REGISTER },
  { "$5.duw", TOK_DOLLAR_DATA_REGISTER },
  { "$6.duw", TOK_DOLLAR_DATA_REGISTER },
  { "$7.duw", TOK_DOLLAR_DATA_REGISTER },
  { "$8.duw", TOK_DOLLAR_DATA_REGISTER },
  { "$9.duw", TOK_DOLLAR_DATA_REGISTER },

  { "$1.dsw", TOK_DOLLAR_DATA_REGISTER },
  { "$2.dsw", TOK_DOLLAR_DATA_REGISTER },
  { "$3.dsw", TOK_DOLLAR_DATA_REGISTER },
  { "$4.dsw", TOK_DOLLAR_DATA_REGISTER },
  { "$5.dsw", TOK_DOLLAR_DATA_REGISTER },
  { "$6.dsw", TOK_DOLLAR_DATA_REGISTER },
  { "$7.dsw", TOK_DOLLAR_DATA_REGISTER },
  { "$8.dsw", TOK_DOLLAR_DATA_REGISTER },
  { "$9.dsw", TOK_DOLLAR_DATA_REGISTER },

  { "$1.dub", TOK_DOLLAR_DATA_REGISTER },
  { "$2.dub", TOK_DOLLAR_DATA_REGISTER },
  { "$3.dub", TOK_DOLLAR_DATA_REGISTER },
  { "$4.dub", TOK_DOLLAR_DATA_REGISTER },
  { "$5.dub", TOK_DOLLAR_DATA_REGISTER },
  { "$6.dub", TOK_DOLLAR_DATA_REGISTER },
  { "$7.dub", TOK_DOLLAR_DATA_REGISTER },
  { "$8.dub", TOK_DOLLAR_DATA_REGISTER },
  { "$9.dub", TOK_DOLLAR_DATA_REGISTER },

  { "$1.dsb", TOK_DOLLAR_DATA_REGISTER },
  { "$2.dsb", TOK_DOLLAR_DATA_REGISTER },
  { "$3.dsb", TOK_DOLLAR_DATA_REGISTER },
  { "$4.dsb", TOK_DOLLAR_DATA_REGISTER },
  { "$5.dsb", TOK_DOLLAR_DATA_REGISTER },
  { "$6.dsb", TOK_DOLLAR_DATA_REGISTER },
  { "$7.dsb", TOK_DOLLAR_DATA_REGISTER },
  { "$8.dsb", TOK_DOLLAR_DATA_REGISTER },
  { "$9.dsb", TOK_DOLLAR_DATA_REGISTER },

  { "$1.aul", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$2.aul", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$3.aul", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$4.aul", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$5.aul", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$6.aul", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$7.aul", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$8.aul", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$9.aul", TOK_DOLLAR_ADDRESS_REGISTER },

  { "$1.asl", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$2.asl", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$3.asl", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$4.asl", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$5.asl", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$6.asl", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$7.asl", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$8.asl", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$9.asl", TOK_DOLLAR_ADDRESS_REGISTER },

  { "$1.auw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$2.auw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$3.auw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$4.auw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$5.auw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$6.auw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$7.auw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$8.auw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$9.auw", TOK_DOLLAR_ADDRESS_REGISTER },

  { "$1.asw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$2.asw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$3.asw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$4.asw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$5.asw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$6.asw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$7.asw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$8.asw", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$9.asw", TOK_DOLLAR_ADDRESS_REGISTER },

  { "$1.aub", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$2.aub", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$3.aub", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$4.aub", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$5.aub", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$6.aub", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$7.aub", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$8.aub", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$9.aub", TOK_DOLLAR_ADDRESS_REGISTER },

  { "$1.asb", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$2.asb", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$3.asb", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$4.asb", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$5.asb", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$6.asb", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$7.asb", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$8.asb", TOK_DOLLAR_ADDRESS_REGISTER },
  { "$9.asb", TOK_DOLLAR_ADDRESS_REGISTER },

  { "$1.gul", TOK_DOLLAR_GENERAL_REGISTER },
  { "$2.gul", TOK_DOLLAR_GENERAL_REGISTER },
  { "$3.gul", TOK_DOLLAR_GENERAL_REGISTER },
  { "$4.gul", TOK_DOLLAR_GENERAL_REGISTER },
  { "$5.gul", TOK_DOLLAR_GENERAL_REGISTER },
  { "$6.gul", TOK_DOLLAR_GENERAL_REGISTER },
  { "$7.gul", TOK_DOLLAR_GENERAL_REGISTER },
  { "$8.gul", TOK_DOLLAR_GENERAL_REGISTER },
  { "$9.gul", TOK_DOLLAR_GENERAL_REGISTER },

  { "$1.gsl", TOK_DOLLAR_GENERAL_REGISTER },
  { "$2.gsl", TOK_DOLLAR_GENERAL_REGISTER },
  { "$3.gsl", TOK_DOLLAR_GENERAL_REGISTER },
  { "$4.gsl", TOK_DOLLAR_GENERAL_REGISTER },
  { "$5.gsl", TOK_DOLLAR_GENERAL_REGISTER },
  { "$6.gsl", TOK_DOLLAR_GENERAL_REGISTER },
  { "$7.gsl", TOK_DOLLAR_GENERAL_REGISTER },
  { "$8.gsl", TOK_DOLLAR_GENERAL_REGISTER },
  { "$9.gsl", TOK_DOLLAR_GENERAL_REGISTER },

  { "$1.guw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$2.guw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$3.guw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$4.guw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$5.guw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$6.guw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$7.guw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$8.guw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$9.guw", TOK_DOLLAR_GENERAL_REGISTER },

  { "$1.gsw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$2.gsw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$3.gsw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$4.gsw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$5.gsw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$6.gsw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$7.gsw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$8.gsw", TOK_DOLLAR_GENERAL_REGISTER },
  { "$9.gsw", TOK_DOLLAR_GENERAL_REGISTER },

  { "$1.gub", TOK_DOLLAR_GENERAL_REGISTER },
  { "$2.gub", TOK_DOLLAR_GENERAL_REGISTER },
  { "$3.gub", TOK_DOLLAR_GENERAL_REGISTER },
  { "$4.gub", TOK_DOLLAR_GENERAL_REGISTER },
  { "$5.gub", TOK_DOLLAR_GENERAL_REGISTER },
  { "$6.gub", TOK_DOLLAR_GENERAL_REGISTER },
  { "$7.gub", TOK_DOLLAR_GENERAL_REGISTER },
  { "$8.gub", TOK_DOLLAR_GENERAL_REGISTER },
  { "$9.gub", TOK_DOLLAR_GENERAL_REGISTER },

  { "$1.gsb", TOK_DOLLAR_GENERAL_REGISTER },
  { "$2.gsb", TOK_DOLLAR_GENERAL_REGISTER },
  { "$3.gsb", TOK_DOLLAR_GENERAL_REGISTER },
  { "$4.gsb", TOK_DOLLAR_GENERAL_REGISTER },
  { "$5.gsb", TOK_DOLLAR_GENERAL_REGISTER },
  { "$6.gsb", TOK_DOLLAR_GENERAL_REGISTER },
  { "$7.gsb", TOK_DOLLAR_GENERAL_REGISTER },
  { "$8.gsb", TOK_DOLLAR_GENERAL_REGISTER },
  { "$9.gsb", TOK_DOLLAR_GENERAL_REGISTER },

  { "$1.mul", TOK_DOLLAR_AMODE },
  { "$2.mul", TOK_DOLLAR_AMODE },
  { "$3.mul", TOK_DOLLAR_AMODE },
  { "$4.mul", TOK_DOLLAR_AMODE },
  { "$5.mul", TOK_DOLLAR_AMODE },
  { "$6.mul", TOK_DOLLAR_AMODE },
  { "$7.mul", TOK_DOLLAR_AMODE },
  { "$8.mul", TOK_DOLLAR_AMODE },
  { "$9.mul", TOK_DOLLAR_AMODE },

  { "$1.msl", TOK_DOLLAR_AMODE },
  { "$2.msl", TOK_DOLLAR_AMODE },
  { "$3.msl", TOK_DOLLAR_AMODE },
  { "$4.msl", TOK_DOLLAR_AMODE },
  { "$5.msl", TOK_DOLLAR_AMODE },
  { "$6.msl", TOK_DOLLAR_AMODE },
  { "$7.msl", TOK_DOLLAR_AMODE },
  { "$8.msl", TOK_DOLLAR_AMODE },
  { "$9.msl", TOK_DOLLAR_AMODE },

  { "$1.muw", TOK_DOLLAR_AMODE },
  { "$2.muw", TOK_DOLLAR_AMODE },
  { "$3.muw", TOK_DOLLAR_AMODE },
  { "$4.muw", TOK_DOLLAR_AMODE },
  { "$5.muw", TOK_DOLLAR_AMODE },
  { "$6.muw", TOK_DOLLAR_AMODE },
  { "$7.muw", TOK_DOLLAR_AMODE },
  { "$8.muw", TOK_DOLLAR_AMODE },
  { "$9.muw", TOK_DOLLAR_AMODE },

  { "$1.msw", TOK_DOLLAR_AMODE },
  { "$2.msw", TOK_DOLLAR_AMODE },
  { "$3.msw", TOK_DOLLAR_AMODE },
  { "$4.msw", TOK_DOLLAR_AMODE },
  { "$5.msw", TOK_DOLLAR_AMODE },
  { "$6.msw", TOK_DOLLAR_AMODE },
  { "$7.msw", TOK_DOLLAR_AMODE },
  { "$8.msw", TOK_DOLLAR_AMODE },
  { "$9.msw", TOK_DOLLAR_AMODE },

  { "$1.mub", TOK_DOLLAR_AMODE },
  { "$2.mub", TOK_DOLLAR_AMODE },
  { "$3.mub", TOK_DOLLAR_AMODE },
  { "$4.mub", TOK_DOLLAR_AMODE },
  { "$5.mub", TOK_DOLLAR_AMODE },
  { "$6.mub", TOK_DOLLAR_AMODE },
  { "$7.mub", TOK_DOLLAR_AMODE },
  { "$8.mub", TOK_DOLLAR_AMODE },
  { "$9.mub", TOK_DOLLAR_AMODE },

  { "$1.msb", TOK_DOLLAR_AMODE },
  { "$2.msb", TOK_DOLLAR_AMODE },
  { "$3.msb", TOK_DOLLAR_AMODE },
  { "$4.msb", TOK_DOLLAR_AMODE },
  { "$5.msb", TOK_DOLLAR_AMODE },
  { "$6.msb", TOK_DOLLAR_AMODE },
  { "$7.msb", TOK_DOLLAR_AMODE },
  { "$8.msb", TOK_DOLLAR_AMODE },
  { "$9.msb", TOK_DOLLAR_AMODE },

  { "$1.rul", TOK_DOLLAR_REVERSED_AMODE },
  { "$2.rul", TOK_DOLLAR_REVERSED_AMODE },
  { "$3.rul", TOK_DOLLAR_REVERSED_AMODE },
  { "$4.rul", TOK_DOLLAR_REVERSED_AMODE },
  { "$5.rul", TOK_DOLLAR_REVERSED_AMODE },
  { "$6.rul", TOK_DOLLAR_REVERSED_AMODE },
  { "$7.rul", TOK_DOLLAR_REVERSED_AMODE },
  { "$8.rul", TOK_DOLLAR_REVERSED_AMODE },
  { "$9.rul", TOK_DOLLAR_REVERSED_AMODE },

  { "$1.rsl", TOK_DOLLAR_REVERSED_AMODE },
  { "$2.rsl", TOK_DOLLAR_REVERSED_AMODE },
  { "$3.rsl", TOK_DOLLAR_REVERSED_AMODE },
  { "$4.rsl", TOK_DOLLAR_REVERSED_AMODE },
  { "$5.rsl", TOK_DOLLAR_REVERSED_AMODE },
  { "$6.rsl", TOK_DOLLAR_REVERSED_AMODE },
  { "$7.rsl", TOK_DOLLAR_REVERSED_AMODE },
  { "$8.rsl", TOK_DOLLAR_REVERSED_AMODE },
  { "$9.rsl", TOK_DOLLAR_REVERSED_AMODE },

  { "$1.ruw", TOK_DOLLAR_REVERSED_AMODE },
  { "$2.ruw", TOK_DOLLAR_REVERSED_AMODE },
  { "$3.ruw", TOK_DOLLAR_REVERSED_AMODE },
  { "$4.ruw", TOK_DOLLAR_REVERSED_AMODE },
  { "$5.ruw", TOK_DOLLAR_REVERSED_AMODE },
  { "$6.ruw", TOK_DOLLAR_REVERSED_AMODE },
  { "$7.ruw", TOK_DOLLAR_REVERSED_AMODE },
  { "$8.ruw", TOK_DOLLAR_REVERSED_AMODE },
  { "$9.ruw", TOK_DOLLAR_REVERSED_AMODE },

  { "$1.rsw", TOK_DOLLAR_REVERSED_AMODE },
  { "$2.rsw", TOK_DOLLAR_REVERSED_AMODE },
  { "$3.rsw", TOK_DOLLAR_REVERSED_AMODE },
  { "$4.rsw", TOK_DOLLAR_REVERSED_AMODE },
  { "$5.rsw", TOK_DOLLAR_REVERSED_AMODE },
  { "$6.rsw", TOK_DOLLAR_REVERSED_AMODE },
  { "$7.rsw", TOK_DOLLAR_REVERSED_AMODE },
  { "$8.rsw", TOK_DOLLAR_REVERSED_AMODE },
  { "$9.rsw", TOK_DOLLAR_REVERSED_AMODE },

  { "$1.rub", TOK_DOLLAR_REVERSED_AMODE },
  { "$2.rub", TOK_DOLLAR_REVERSED_AMODE },
  { "$3.rub", TOK_DOLLAR_REVERSED_AMODE },
  { "$4.rub", TOK_DOLLAR_REVERSED_AMODE },
  { "$5.rub", TOK_DOLLAR_REVERSED_AMODE },
  { "$6.rub", TOK_DOLLAR_REVERSED_AMODE },
  { "$7.rub", TOK_DOLLAR_REVERSED_AMODE },
  { "$8.rub", TOK_DOLLAR_REVERSED_AMODE },
  { "$9.rub", TOK_DOLLAR_REVERSED_AMODE },

  { "$1.rsb", TOK_DOLLAR_REVERSED_AMODE },
  { "$2.rsb", TOK_DOLLAR_REVERSED_AMODE },
  { "$3.rsb", TOK_DOLLAR_REVERSED_AMODE },
  { "$4.rsb", TOK_DOLLAR_REVERSED_AMODE },
  { "$5.rsb", TOK_DOLLAR_REVERSED_AMODE },
  { "$6.rsb", TOK_DOLLAR_REVERSED_AMODE },
  { "$7.rsb", TOK_DOLLAR_REVERSED_AMODE },
  { "$8.rsb", TOK_DOLLAR_REVERSED_AMODE },
  { "$9.rsb", TOK_DOLLAR_REVERSED_AMODE },

  { "$1.pul", TOK_DOLLAR_AMODE_PTR },
  { "$2.pul", TOK_DOLLAR_AMODE_PTR },
  { "$3.pul", TOK_DOLLAR_AMODE_PTR },
  { "$4.pul", TOK_DOLLAR_AMODE_PTR },
  { "$5.pul", TOK_DOLLAR_AMODE_PTR },
  { "$6.pul", TOK_DOLLAR_AMODE_PTR },
  { "$7.pul", TOK_DOLLAR_AMODE_PTR },
  { "$8.pul", TOK_DOLLAR_AMODE_PTR },
  { "$9.pul", TOK_DOLLAR_AMODE_PTR },

  { "$1.psl", TOK_DOLLAR_AMODE_PTR },
  { "$2.psl", TOK_DOLLAR_AMODE_PTR },
  { "$3.psl", TOK_DOLLAR_AMODE_PTR },
  { "$4.psl", TOK_DOLLAR_AMODE_PTR },
  { "$5.psl", TOK_DOLLAR_AMODE_PTR },
  { "$6.psl", TOK_DOLLAR_AMODE_PTR },
  { "$7.psl", TOK_DOLLAR_AMODE_PTR },
  { "$8.psl", TOK_DOLLAR_AMODE_PTR },
  { "$9.psl", TOK_DOLLAR_AMODE_PTR },

  { "$1.puw", TOK_DOLLAR_AMODE_PTR },
  { "$2.puw", TOK_DOLLAR_AMODE_PTR },
  { "$3.puw", TOK_DOLLAR_AMODE_PTR },
  { "$4.puw", TOK_DOLLAR_AMODE_PTR },
  { "$5.puw", TOK_DOLLAR_AMODE_PTR },
  { "$6.puw", TOK_DOLLAR_AMODE_PTR },
  { "$7.puw", TOK_DOLLAR_AMODE_PTR },
  { "$8.puw", TOK_DOLLAR_AMODE_PTR },
  { "$9.puw", TOK_DOLLAR_AMODE_PTR },

  { "$1.psw", TOK_DOLLAR_AMODE_PTR },
  { "$2.psw", TOK_DOLLAR_AMODE_PTR },
  { "$3.psw", TOK_DOLLAR_AMODE_PTR },
  { "$4.psw", TOK_DOLLAR_AMODE_PTR },
  { "$5.psw", TOK_DOLLAR_AMODE_PTR },
  { "$6.psw", TOK_DOLLAR_AMODE_PTR },
  { "$7.psw", TOK_DOLLAR_AMODE_PTR },
  { "$8.psw", TOK_DOLLAR_AMODE_PTR },
  { "$9.psw", TOK_DOLLAR_AMODE_PTR },

  { "$1.pub", TOK_DOLLAR_AMODE_PTR },
  { "$2.pub", TOK_DOLLAR_AMODE_PTR },
  { "$3.pub", TOK_DOLLAR_AMODE_PTR },
  { "$4.pub", TOK_DOLLAR_AMODE_PTR },
  { "$5.pub", TOK_DOLLAR_AMODE_PTR },
  { "$6.pub", TOK_DOLLAR_AMODE_PTR },
  { "$7.pub", TOK_DOLLAR_AMODE_PTR },
  { "$8.pub", TOK_DOLLAR_AMODE_PTR },
  { "$9.pub", TOK_DOLLAR_AMODE_PTR },

  { "$1.psb", TOK_DOLLAR_AMODE_PTR },
  { "$2.psb", TOK_DOLLAR_AMODE_PTR },
  { "$3.psb", TOK_DOLLAR_AMODE_PTR },
  { "$4.psb", TOK_DOLLAR_AMODE_PTR },
  { "$5.psb", TOK_DOLLAR_AMODE_PTR },
  { "$6.psb", TOK_DOLLAR_AMODE_PTR },
  { "$7.psb", TOK_DOLLAR_AMODE_PTR },
  { "$8.psb", TOK_DOLLAR_AMODE_PTR },
  { "$9.psb", TOK_DOLLAR_AMODE_PTR },

  { "$1.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$2.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$3.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$4.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$5.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$6.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$7.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$8.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$9.qul", TOK_DOLLAR_REVERSED_AMODE_PTR },

  { "$1.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$2.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$3.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$4.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$5.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$6.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$7.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$8.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$9.qsl", TOK_DOLLAR_REVERSED_AMODE_PTR },

  { "$1.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$2.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$3.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$4.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$5.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$6.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$7.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$8.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$9.quw", TOK_DOLLAR_REVERSED_AMODE_PTR },

  { "$1.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$2.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$3.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$4.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$5.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$6.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$7.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$8.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$9.qsw", TOK_DOLLAR_REVERSED_AMODE_PTR },

  { "$1.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$2.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$3.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$4.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$5.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$6.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$7.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$8.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$9.qub", TOK_DOLLAR_REVERSED_AMODE_PTR },

  { "$1.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$2.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$3.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$4.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$5.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$6.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$7.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$8.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },
  { "$9.qsb", TOK_DOLLAR_REVERSED_AMODE_PTR },

  { "$1.ub", TOK_DOLLAR_NUMBER },
  { "$2.ub", TOK_DOLLAR_NUMBER },
  { "$3.ub", TOK_DOLLAR_NUMBER },
  { "$4.ub", TOK_DOLLAR_NUMBER },
  { "$5.ub", TOK_DOLLAR_NUMBER },
  { "$6.ub", TOK_DOLLAR_NUMBER },
  { "$7.ub", TOK_DOLLAR_NUMBER },
  { "$8.ub", TOK_DOLLAR_NUMBER },
  { "$9.ub", TOK_DOLLAR_NUMBER },

  { "$1.sb", TOK_DOLLAR_NUMBER },
  { "$2.sb", TOK_DOLLAR_NUMBER },
  { "$3.sb", TOK_DOLLAR_NUMBER },
  { "$4.sb", TOK_DOLLAR_NUMBER },
  { "$5.sb", TOK_DOLLAR_NUMBER },
  { "$6.sb", TOK_DOLLAR_NUMBER },
  { "$7.sb", TOK_DOLLAR_NUMBER },
  { "$8.sb", TOK_DOLLAR_NUMBER },
  { "$9.sb", TOK_DOLLAR_NUMBER },

  { "$1.uw", TOK_DOLLAR_NUMBER },
  { "$2.uw", TOK_DOLLAR_NUMBER },
  { "$3.uw", TOK_DOLLAR_NUMBER },
  { "$4.uw", TOK_DOLLAR_NUMBER },
  { "$5.uw", TOK_DOLLAR_NUMBER },
  { "$6.uw", TOK_DOLLAR_NUMBER },
  { "$7.uw", TOK_DOLLAR_NUMBER },
  { "$8.uw", TOK_DOLLAR_NUMBER },
  { "$9.uw", TOK_DOLLAR_NUMBER },

  { "$1.sw", TOK_DOLLAR_NUMBER },
  { "$2.sw", TOK_DOLLAR_NUMBER },
  { "$3.sw", TOK_DOLLAR_NUMBER },
  { "$4.sw", TOK_DOLLAR_NUMBER },
  { "$5.sw", TOK_DOLLAR_NUMBER },
  { "$6.sw", TOK_DOLLAR_NUMBER },
  { "$7.sw", TOK_DOLLAR_NUMBER },
  { "$8.sw", TOK_DOLLAR_NUMBER },
  { "$9.sw", TOK_DOLLAR_NUMBER },

  { "$1.ul", TOK_DOLLAR_NUMBER },
  { "$2.ul", TOK_DOLLAR_NUMBER },
  { "$3.ul", TOK_DOLLAR_NUMBER },
  { "$4.ul", TOK_DOLLAR_NUMBER },
  { "$5.ul", TOK_DOLLAR_NUMBER },
  { "$6.ul", TOK_DOLLAR_NUMBER },
  { "$7.ul", TOK_DOLLAR_NUMBER },
  { "$8.ul", TOK_DOLLAR_NUMBER },
  { "$9.ul", TOK_DOLLAR_NUMBER },

  { "$1.sl", TOK_DOLLAR_NUMBER },
  { "$2.sl", TOK_DOLLAR_NUMBER },
  { "$3.sl", TOK_DOLLAR_NUMBER },
  { "$4.sl", TOK_DOLLAR_NUMBER },
  { "$5.sl", TOK_DOLLAR_NUMBER },
  { "$6.sl", TOK_DOLLAR_NUMBER },
  { "$7.sl", TOK_DOLLAR_NUMBER },
  { "$8.sl", TOK_DOLLAR_NUMBER },
  { "$9.sl", TOK_DOLLAR_NUMBER },
};
